import { type EmbeddedWalletConstructorType, type GetUser } from "../interfaces/embedded-wallets/embedded-wallets";
import { EmbeddedWalletIframeCommunicator } from "../utils/iFrameCommunication/EmbeddedWalletIframeCommunicator";
import type { AuthQuerierTypes } from "./auth";
import { Auth } from "./auth";
export declare class EmbeddedWalletSdk {
    protected clientId: string;
    protected querier: EmbeddedWalletIframeCommunicator<AuthQuerierTypes>;
    private wallet;
    /**
     * Used to manage the Auth state of the user.
     */
    auth: Auth;
    private isClientIdLegacyPaper;
    /**
     * @example
     * `const thirdwebEmbeddedWallet = new EmbeddedWalletSdk({ clientId: "", chain: "Goerli" });`
     * @param param0 - EmbeddedWalletConstructorType
     */
    constructor({ clientId, chain, styles, onAuthSuccess, }: EmbeddedWalletConstructorType);
    /**
     * Gets the usr if they are logged in
     * @example
     * ```js
     *  const user = await thirdwebEmbeddedWallet.getUser();
     *  switch (user.status) {
     *     case UserWalletStatus.LOGGED_OUT: {
     *       // User is logged out, call one of the auth methods on thirdwebEmbeddedWallet.auth to authenticate the user
     *       break;
     *     }
     *     case UserWalletStatus.LOGGED_IN_WALLET_INITIALIZED: {
     *       // user is logged in and wallet is all set up.
     *       // You have access to:
     *       user.status;
     *       user.authDetails;
     *       user.walletAddress;
     *       user.wallet;
     *       break;
     *     }
     * }
     * ```
     * @returns GetUser - an object to containing various information on the user statuses
     */
    getUser(): Promise<GetUser>;
}
//# sourceMappingURL=embedded-wallet.d.ts.map